@@ -1,92 +1,85 @@
 import abc
 
-class NodeBase(object):
-    """A node in the network."""
 
-    @property
-    def node_id(self):
-        """A string that uniquely identifies this node in the network."""
-        raise NotImplementedError()
+class NodeBase(object):
+    """A node in the network.
+
+    :ivar node_id: A string that uniquely identifies this node in the network.
+
+    """
+
+    def __init__(self):
+        self.node_id = None
 
 
 class ActivityObjectBase(object):
-    """A thing which participates in an Activity."""
+    """A thing which participates in an Activity.
 
-    @property
-    def activity_name(self):
-        """Unicode representation of this object."""
-        raise NotImplementedError()
+    :ivar activity_name str:
+        Unicode representation of this object.
 
-    @property
-    def activity_url(self):
-        """URL of this object."""
-        raise NotImplementedError()
+    :ivar activity_url str:
+        URL of this object.
 
-    @property
-    def activity_extras(self):
-        """A BSON-serializable dict of extra stuff to store on the activity.
-        """
-        return {}
+    :ivar activity_extras dict:
+        A BSON-serializable dict of extra stuff to store on the activity.
+
+    """
+
+    def __init__(self):
+        self.activity_name = None
+        self.activity_url = None
+        self.activity_extras = {}
 
 
 class ActivityBase(object):
     """Tells the story of a person performing an action on or with an object.
 
     Consists of an actor, a verb, an object, and optionally a target.
-    """
-    __metaclass__ = abc.ABCMeta
 
-    @abc.abstractproperty
-    def actor(self):
-        """The actor, or subject, of the Activity.
+    Defines the following attributes:
+
+    :ivar actor:
+        The actor, or subject, of the Activity.
 
         Example: *John* posted a comment on ticket #42.
 
-        Returns:
-            :class:`NodeBase`
-        """
-        return None
+        :class:`NodeBase, ActivityObjectBase`
 
-    @abc.abstractproperty
-    def verb(self):
-        """The verb in the Activity.
+    :ivar verb:
+        The verb in the Activity.
 
         Example: John *posted* a comment on ticket #42.
 
-        Returns: str
-        """
-        return None
+        :class:`str`
 
-    @abc.abstractproperty
-    def obj(self):
-        """The object of the Activity.
+    :ivar obj:
+        The object of the Activity.
 
         Example: John posted *a comment* on ticket #42.
 
-        Returns:
-            :class:`ObjectBase`
-        """
-        return None
+        :class:`ActivityObjectBase`
 
-    @abc.abstractproperty
-    def target(self):
-        """The (optional) target of the Activity.
+    :ivar target:
+        The (optional) target of the Activity.
 
         Example: John posted a comment on *ticket #42*.
 
-        Returns:
-            :class:`ObjectBase`
-        """
-        return None
+        :class:`ActivityObjectBase`
 
-    @abc.abstractproperty
-    def published(self):
-        """The datetime at which the Activity was published.
+    :ivar published:
+        The datetime at which the Activity was published.
 
-        Returns:
-            :class:`datetime.datetime`
-        """
-        return None
+        :class:`datetime.datetime`
+
+    """
+
+    def __init__(self):
+        self.actor = None
+        self.verb = None
+        self.obj = None
+        self.target = None
+        self.published = None
 
 
 class NodeManagerBase(object):
