@@ -1,66 +1,66 @@
 import datetime
 import logging
 import time
-from operator import itemgetter
+from operator import attrgetter
 
-from activitystream import base
+from . import base
+from .storage.base import (
+        StoredActivity,
+        ActivityObject,
+        )
 
 log = logging.getLogger(__name__)
 
+
 class NodeManager(base.NodeManagerBase):
     """Manages the network of connected nodes.
 
     Knows how to connect and disconnect nodes and serialize the graph.
     """
 
-    def __init__(self, coll):
-        self.coll = coll
+    def __init__(self, storage):
+        self.storage = storage
 
     def follow(self, follower, following):
         """Create a directed edge from :class:`Node` ``follower`` to
         :class:`Node` ``following``.
         """
-        self.coll.update({"node_id": follower.node_id},
-                {"$addToSet": {"following": following.node_id}}, upsert=True)
-        self.coll.update({"node_id": following.node_id},
-                {"$addToSet": {"followers": follower.node_id}}, upsert=True)
+        self.storage.create_edge(follower, following)
 
     def unfollow(self, follower, following):
         """Destroy a directed edge from :class:`Node` ``follower`` to
         :class:`Node` ``following``.
         """
-        self.coll.update({"node_id": follower.node_id},
-                {"$pull": {"following": following.node_id}})
-        self.coll.update({"node_id": following.node_id},
-                {"$pull": {"followers": follower.node_id}})
+        self.storage.destroy_edge(follower, following)
 
     def is_following(self, follower, following):
         """Determine if there is a directed edge from :class:`Node`
         ``follower`` to :class:`Node` ``following``.
         """
-        result = self.coll.find_one({"node_id": follower.node_id,
-                "following": following.node_id})
-        return result is not None
+        return self.storage.edge_exists(follower, following)
 
     def get_node(self, node_id):
         """Return the node for the given node_id."""
-        return self.coll.find_one({"node_id": node_id})
+        return self.storage.get_node(node_id)
 
     def get_nodes(self, node_ids):
         """Return nodes for the given node_ids."""
-        return list(self.coll.find({"node_id": {"$in": node_ids}}))
+        return self.storage.get_nodes(node_ids)
+
+    def create_node(self, node_id):
+        """Create a new node"""
+        return self.storage.create_node(node_id)
 
     def save_node(self, node):
-        """Save (upsert) a node into the collection."""
-        self.coll.save(node)
-        return node
+        """Save an existing node"""
+        return self.storage.save_node(node)
 
 
 class ActivityManager(base.ActivityManagerBase):
     """Serializes :class:`Activity` objects."""
 
-    def __init__(self, activity_coll, node_manager):
-        self.activity_coll = activity_coll
+    def __init__(self, storage, node_manager):
+        self.storage = storage
         self.node_manager = node_manager
 
     def create(self, actor, verb, obj, target=None, related_nodes=None):
@@ -69,20 +69,6 @@ class ActivityManager(base.ActivityManagerBase):
         Serializing includes making a copy of the activity for any node in the
         network that is connected to any node in the activity.
         """
-        def obj_dict(obj):
-            d = dict(activity_name=obj.activity_name,
-                     activity_url=obj.activity_url)
-            if obj.activity_extras:
-                d['activity_extras'] = obj.activity_extras
-            return d
-
-        # Create base activity
-        activity = {"actor": obj_dict(actor), "verb": verb, "obj": obj_dict(obj),
-                "target": None, "published": datetime.datetime.utcnow()}
-        activity["actor"]["node_id"] = actor.node_id
-        if target:
-            activity["target"] = obj_dict(target)
-
         # Figure out who needs a copy
         related_nodes = related_nodes or []
         owners = [
@@ -90,9 +76,15 @@ class ActivityManager(base.ActivityManagerBase):
                 if hasattr(node, 'node_id')]
 
         for owner in owners:
-            instance = {"node_id": owner}
-            instance.update(activity)
-            self.activity_coll.insert(instance)
+            activity = StoredActivity(
+                    actor=actor,
+                    verb=verb,
+                    obj=obj,
+                    target=target,
+                    published=datetime.datetime.utcnow(),
+                    node_id = owner
+                    )
+            self.storage.save_activity(activity)
         return activity
 
     def get_activities(self, nodes, since=None, sort=None, limit=0, skip=0, query=None):
@@ -102,54 +94,44 @@ class ActivityManager(base.ActivityManagerBase):
             since (datetime) - return activities that have occured since this
                                datetime
         """
-        node_ids = [node["node_id"] for node in nodes]
-        q = {'node_id': {'$in': node_ids}}
-        if since:
-            q['published'] = {'$gt': since}
-        if query:
-            q.update(query)
-        it = self.activity_coll.find(q, sort=sort, limit=limit, skip=skip)
-        return list(it)
+        return self.storage.get_activities(nodes, since=since, sort=sort,
+                limit=limit, skip=skip, query=query)
 
-    def save_activities(self, activities):
-        """Save (upsert) a list of activities to mongo."""
-        for activity in activities:
-            self.activity_coll.save(activity)
+    def save_timeline(self, node_id, activities):
+        """Save a list of activities to a node's timeline."""
+        self.storage.save_timeline(node_id, activities)
 
     def get_timeline(self, node_id, sort=None, limit=0, skip=0, query=None):
         """Return the timeline for node_id.
 
-        Timeline is the already-aggregated list of activities in mongo.
+        Timeline is the already-aggregated list of activities.
         """
-        q = {'owner_id': node_id}
-        if query:
-            q.update(query)
-        it = self.activity_coll.find(q, sort=sort, limit=limit, skip=skip)
-        return list(it)
+        return self.storage.get_timeline(node_id, sort=sort, limit=limit,
+                skip=skip, query=query)
 
 
 class Aggregator(object):
     """Creates a timeline for a given node in the network graph."""
 
-    def __init__(self, activity_manager, timeline_manager, node_manager):
+    def __init__(self, activity_manager, node_manager):
         self.node_manager = node_manager
         self.activity_manager = activity_manager
-        self.timeline_manager = timeline_manager
 
     def _unique_activities(self, activities):
-        """Return a list of unique activities. Can't use set() b/c activities
-        are dicts and therefore unhashable.
+        """Return a list of unique activities.
+
         """
-        ignore_keys = ('score', 'flags', '_id', 'owner_id', 'node_id')
-        unique_orig, unique_norm  = [], []
-        for orig in activities:
-            norm = orig.copy()
-            for k in ignore_keys:
-                if k in norm: del norm[k]
-            if norm in unique_norm: continue
-            unique_norm.append(norm)
-            unique_orig.append(orig)
-        return unique_orig
+        attrs = ('actor', 'obj', 'target')
+        seen, unique = [], []
+        for activity in activities:
+            d = dict((a, ActivityObject.to_dict(getattr(activity, a)))
+                    for a in attrs)
+            d.update(verb=activity.verb)
+            if d in seen:
+                continue
+            seen.append(d)
+            unique.append(activity)
+        return unique
 
     def classify_activities(self, activities):
         """Return a list of activities with classfication flags added."""
@@ -161,12 +143,14 @@ class Aggregator(object):
         node = self.node_manager.get_node(node_id)
         if not node:
             log.warning('Node not found for node_id "%s". Creating...', node_id)
-            node = self.node_manager.save_node(dict(node_id=node_id))
-        last_timeline_aggregation = node.get('last_timeline_aggregation')
-        node['last_timeline_aggregation'] = datetime.datetime.utcnow()
+            node = self.node_manager.create_node(node_id)
+        last_timeline_aggregation = node.last_timeline_aggregation
+        node.last_timeline_aggregation = datetime.datetime.utcnow()
+        # save node to persist last_timeline_aggregation timestamp
+        self.node_manager.save_node(node)
         # get a subset of the nodes be followed
         connections = self.filter_connections(
-                self.node_manager.get_nodes(node.get('following', [])))
+                self.node_manager.get_nodes(node.following or []))
         # retrieve the followed nodes' activities
         activities = self.activity_manager.get_activities(connections,
                 since=last_timeline_aggregation)
@@ -178,28 +162,15 @@ class Aggregator(object):
         # if we don't have any activities at this point, there's nothing from
         # which to generate a timeline
         if not activities:
-            return []
-        # strip out _ids from activities collection, since they don't
-        # apply to the timeline collection
-        for activity in activities:
-            activity.pop('_id', None)
-        # add historical timeline activities
-        activities += self.timeline_manager.get_timeline(node_id)
+            return self.activity_manager.get_timeline(node_id)
         # remove duplicates
         activities = self._unique_activities(activities)
         # classify and score activities
         activities = self.classify_activities(activities)
         activities = self.score_activities(activities)
-        # sort and truncate the timeline
-        activities = sorted(activities, key=itemgetter('score'), reverse=True)
-        activities = self.truncate_timeline(activities)
         # save to this node's timeline
-        for a in activities:
-            a['owner_id'] = node_id
-        self.timeline_manager.save_activities(activities)
-        # save node to persist last_timeline_aggregation timestamp
-        self.node_manager.save_node(node)
-        return activities
+        self.activity_manager.save_timeline(node_id, activities)
+        return self.activity_manager.get_timeline(node_id)
 
     def needs_aggregation(self, node):
         """Return True if it's time for this node's timeline to be
@@ -209,7 +180,7 @@ class Aggregator(object):
         return True
 
         # Alternative, based on time since last aggregation:
-        last = node.get('last_timeline_aggregation')
+        last = node.last_timeline_aggregation
         if not last: return True
         # TODO: put this time threshhold in config
         return (datetime.datetime.utcnow() - last).seconds > 60
@@ -232,12 +203,12 @@ class Aggregator(object):
         """
         node_id = node.node_id
         node = self.node_manager.get_node(node_id)
-        page, limit = int(page), int(limit)
+        page, limit = int(page), int(limit or 0)
         if not node or self.needs_aggregation(node):
             self.create_timeline(node_id)
         query_filter = {'actor.node_id': node_id} if actor_only else None
-        timeline = self.timeline_manager.get_timeline(
-                node_id, sort=[('published', -1)], skip=page*limit, limit=limit,
+        timeline = self.activity_manager.get_timeline(
+                node_id, sort=[('score', -1)], skip=page*limit, limit=limit,
                 query=query_filter)
         if filter_func:
             timeline = filter(filter_func, timeline)
@@ -260,11 +231,5 @@ class Aggregator(object):
         have higher scores.
         """
         for a in activities:
-            a['score'] = 1 - (1.0 / time.mktime(a['published'].timetuple()))
-        return activities
-
-    def truncate_timeline(self, activities):
-        """Return a truncated timeline using the algorithm of your choice. The
-        timeline activities passed in are already sorted by score, descending.
-        """
+            a.score = time.mktime(a.published.timetuple())
         return activities
